import click
from scales_nlp import train as run_train
from scales_nlp.utils import DEFAULT_TRAINING_ARGS
import pandas as pd

@click.command()
@click.argument('data-path')
@click.argument('output-dir')
@click.argument('task')
@click.option('--model-name', default=DEFAULT_TRAINING_ARGS['model_name'], help='Huggingface model name')
@click.option('--loss', default=None, help='Name of loss to use for training')
@click.option('--metric', default=None, help='Name of metric to use for evaluation')
@click.option('--max-length', default=DEFAULT_TRAINING_ARGS['max_length'], help='Truncate inputs to max token sequence length')
@click.option('--eval-split', default=DEFAULT_TRAINING_ARGS['eval_split'], help='Proportion of data to use for evaluation')
@click.option('--epochs', default=DEFAULT_TRAINING_ARGS['epochs'], help='Number of training epochs')
@click.option('--train-batch-size', default=DEFAULT_TRAINING_ARGS['train_batch_size'], help='Train batch size')
@click.option('--eval-batch-size', default=DEFAULT_TRAINING_ARGS['eval_batch_size'], help='Evaluation batch size')
@click.option('--gradient-accumulation-steps', default=DEFAULT_TRAINING_ARGS['gradient_accumulation_steps'], help='Artificially increase the train batch size')
@click.option('--learning-rate', default=DEFAULT_TRAINING_ARGS['learning_rate'], help='Learning rate')
@click.option('--warmup-ratio', default=DEFAULT_TRAINING_ARGS['warmup_ratio'], help='Learning rate warmup')
@click.option('--weight-decay', default=DEFAULT_TRAINING_ARGS['weight_decay'], help='Weight decay for AdamW')
@click.option('--save-steps', default=DEFAULT_TRAINING_ARGS['save_steps'], help='Save model checkpoint every n steps')
@click.option('--push', default=None, help='model id to push to hub')
@click.option('--overwrite/--no-overwrite', default=False, help='Overwrite output dir if it exists')
@click.option('--multi-label-delimiter', default='|', help='Delimiter for splitting labels in multi-label-classification task')
@click.option('--text-col', default='text', help='The column with text')
@click.option('--label-col', default='label', help='The column with labels')
def train(
        data_path, output_dir, task, model_name, 
        loss, metric, max_length, eval_split, epochs, 
        train_batch_size, eval_batch_size, gradient_accumulation_steps,
        learning_rate, warmup_ratio, weight_decay,
        save_steps, push, overwrite, 
        multi_label_delimiter, text_col, label_col):
    
    data = pd.read_csv(data_path)
    texts = list(data[text_col].values)
    labels = list(data[label_col].astype(str).values)
    if task == 'multi-label-classification':
        labels = [[x for x in labels.split(multi_label_delimiter) if x != ''] for labels in labels]
        
    run_train(
        output_dir, texts, labels, 
        model_name=model_name, task=task, loss=loss, metric=metric, 
        max_length=max_length, eval_split=eval_split, epochs=epochs,
        train_batch_size=train_batch_size, eval_batch_size=eval_batch_size, 
        gradient_accumulation_steps=gradient_accumulation_steps, learning_rate=learning_rate,
        warmup_ratio=warmup_ratio, weight_decay=weight_decay,
        save_steps=save_steps, push=push, overwrite=overwrite,
    )

if __name__ == '__main__':
    train()