from datasets import load_metric
import torch
from sklearn.metrics import f1_score as sk_f1_score
import numpy as np

acc_metric = load_metric("accuracy")
f1_metric = load_metric("f1")
precision_metric = load_metric("precision")
recall_metric = load_metric("recall")

def multi_class_f1_metric(eval_pred):
    logits, labels = eval_pred
    logits = torch.softmax(torch.Tensor(logits), dim=-1).numpy()
    predictions = np.argmax(logits, axis=-1)
    labels = np.argmax(labels, axis=-1)
    return {
        'accuracy': acc_metric.compute(predictions=predictions, references=labels)['accuracy'],
        'f1': f1_metric.compute(predictions=predictions, references=labels)['f1'],
        'precision': precision_metric.compute(predictions=predictions, references=labels)['precision'],
        'recall': recall_metric.compute(predictions=predictions, references=labels)['recall'],
    }

def multi_label_f1_metric(eval_pred):
    logits, labels = eval_pred
    logits = torch.sigmoid(torch.Tensor(logits)).numpy()
    predictions = (logits > 0.5).astype(int)
    scores = sk_f1_score(predictions, labels, average=None)
    scores = {'labels': {i: scores[i] for i in range(len(scores))}}
    scores['f1_macro'] = sk_f1_score(predictions, labels, average='macro')
    return scores


metrics = {
    'multi_class_f1_metric': multi_class_f1_metric,
    'multi_label_f1_metric': multi_label_f1_metric,
}

